/*
 * Copyright © 2006-2009 Spotify AB
 */
#ifndef _NGX_HTTP_UPSTREAM_SPDHT_MODULE_H_INCLUDED_
#define _NGX_HTTP_UPSTREAM_SPDHT_MODULE_H_INCLUDED_


/* Must be at the top to have feature macros set */
#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_http.h>


/* --- Types --- */
typedef struct {
	time_t tokens_tod;
	ngx_url_t uri;
	ngx_int_t new_server; /* Temporary storage for validate_domain_state() */

	ngx_int_t current_weight;
	ngx_int_t weight;

	ngx_int_t fails;
	time_t accessed;
	ngx_int_t fail_probe;

	ngx_int_t max_fails;
	time_t fail_timeout;

} ngx_http_upstream_spdht_server_t;

typedef struct {
	ngx_str_t token; /* Not ASCII */
	ngx_http_upstream_spdht_server_t *server;
} ngx_http_upstream_spdht_token_t;

typedef struct {
	ngx_str_t name;
	ngx_int_t num_slaves;
} ngx_http_upstream_spdht_collection_t;

typedef struct {
	time_t tod;
	ngx_int_t num_slaves;
	ngx_array_t servers; /* ngx_http_upstream_spdht_server_t, sorted on (host, port) */
	ngx_array_t tokens; /* ngx_http_upstream_spdht_token_t, sorted on token */
	ngx_array_t collections; /* ngx_http_upstream_spdht_collection_t */
	ngx_http_upstream_spdht_server_t *me;
} ngx_http_upstream_spdht_domain_state_t;

typedef struct {
	ngx_str_t name;
	ngx_http_upstream_spdht_domain_state_t state;
} ngx_http_upstream_spdht_domain_t;

typedef struct {
	ngx_str_t srv_prefix;
	ngx_http_upstream_spdht_domain_t domain;
	ngx_array_t parents; /* ngx_http_upstream_spdht_domain_t */
} ngx_http_upstream_spdht_srv_conf_t;

typedef struct {
	ngx_http_handler_pt orig_handler;
	ngx_uint_t max_hops;
	ngx_int_t via_slaves : 1;
	ngx_int_t handle_purge : 1;
	ngx_int_t handle_put : 1;
} ngx_http_upstream_spdht_loc_conf_t;


/* --- Functions --- */
/* In dht.c */
extern ngx_int_t ngx_http_upstream_spdht_master_slaves(ngx_http_upstream_spdht_domain_t *domain, ngx_str_t master_hash, ngx_int_t n, ngx_array_t *dest /* (ngx_http_upstream_spdht_server_t *) */, ngx_log_t *log);

/* In dns.c */
extern char *    ngx_http_upstream_spdht_dns_init(ngx_conf_t *cf);
extern ngx_int_t ngx_http_upstream_spdht_validate_domain_state(ngx_str_t *srv_prefix, ngx_http_upstream_spdht_domain_t *domain, ngx_str_t *server_name, ngx_log_t *log);
extern ngx_int_t ngx_http_upstream_spdht_validate_server(ngx_http_upstream_spdht_server_t *server, ngx_pool_t *pool, ngx_log_t *log);

/* In failover.c */
extern ngx_http_upstream_spdht_server_t *ngx_http_upstream_spdht_failover_server(ngx_http_upstream_spdht_domain_t *domain, ngx_array_t *servers, ngx_log_t *log);
extern void                              ngx_http_upstream_spdht_fail_server(ngx_http_upstream_spdht_server_t *server, ngx_log_t *log);


/* In util.c */
extern ngx_int_t ngx_http_upstream_spdht_bisect_left(const void *rock, const void *_base, size_t nmemb, size_t size, int (*compar)(const void *, const void *));
extern ngx_int_t ngx_http_upstream_spdht_calculate_hash(ngx_str_t *dest, const ngx_http_request_t *r);
extern void      ngx_http_upstream_spdht_domain_dump(const ngx_http_upstream_spdht_domain_t *d, ngx_log_t *log);
extern ngx_int_t ngx_http_upstream_spdht_parse_hex_string(ngx_pool_t *pool, ngx_str_t *dest, const ngx_str_t *src);
extern int       ngx_http_upstream_spdht_token_cmp(const void *a, const void *b);

#endif /* _NGX_HTTP_UPSTREAM_SPDHT_MODULE_H_INCLUDED_ */
